@@ -1,8 +1,9 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 /* exported init enable disable */
 
-const { Atk, Clutter, Gio, GLib, GMenu,
-        GObject, Gtk, Meta, Shell, St } = imports.gi;
+const {
+    Atk, Clutter, Gio, GLib, GMenu, GObject, Gtk, Meta, Shell, St
+} = imports.gi;
 const DND = imports.ui.dnd;
 const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
@@ -21,11 +22,14 @@ const HORIZ_FACTOR = 5;
 const MENU_HEIGHT_OFFSET = 132;
 const NAVIGATION_REGION_OVERSHOOT = 50;
 
+Gio._promisify(Gio._LocalFilePrototype, 'query_info_async', 'query_info_finish');
+Gio._promisify(Gio._LocalFilePrototype, 'set_attributes_async', 'set_attributes_finish');
+
 class ActivitiesMenuItem extends PopupMenu.PopupBaseMenuItem {
     constructor(button) {
         super();
         this._button = button;
-        this.actor.add_child(new St.Label({ text: _("Activities Overview") }));
+        this.actor.add_child(new St.Label({ text: _('Activities Overview') }));
     }
 
     activate(event) {
@@ -44,8 +48,11 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._iconBin = new St.Bin();
         this.actor.add_child(this._iconBin);
 
-        let appLabel = new St.Label({ text: app.get_name(), y_expand: true,
-                                      y_align: Clutter.ActorAlign.CENTER });
+        let appLabel = new St.Label({
+            text: app.get_name(),
+            y_expand: true,
+            y_align: Clutter.ActorAlign.CENTER
+        });
         this.actor.add_child(appLabel);
         this.actor.label_actor = appLabel;
 
@@ -111,7 +118,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         if (this._category)
             name = this._category.get_name();
         else
-            name = _("Favorites");
+            name = _('Favorites');
 
         this.actor.add_child(new St.Label({ text: name }));
         this.actor.connect('motion-event', this._onMotionEvent.bind(this));
@@ -299,48 +306,32 @@ class DesktopTarget {
         return source._app.app_info;
     }
 
-    _touchFile(file) {
-        let queryFlags = Gio.FileQueryInfoFlags.NONE;
-        let ioPriority = GLib.PRIORITY_DEFAULT;
-
-        let info = new Gio.FileInfo();
-        info.set_attribute_uint64(Gio.FILE_ATTRIBUTE_TIME_ACCESS,
-                                  GLib.get_real_time());
-        file.set_attributes_async (info, queryFlags, ioPriority, null,
-            (o, res) => {
-                try {
-                    o.set_attributes_finish(res);
-                } catch (e) {
-                    log(`Failed to update access time: ${e.message}`);
-                }
-            });
-    }
-
-    _markTrusted(file) {
+    async _markTrusted(file) {
         let modeAttr = Gio.FILE_ATTRIBUTE_UNIX_MODE;
         let trustedAttr = 'metadata::trusted';
         let queryFlags = Gio.FileQueryInfoFlags.NONE;
         let ioPriority = GLib.PRIORITY_DEFAULT;
 
-        file.query_info_async(modeAttr, queryFlags, ioPriority, null,
-            (o, res) => {
-                try {
-                    let info = o.query_info_finish(res);
-                    let mode = info.get_attribute_uint32(modeAttr) | 0o100;
+        try {
+            let info = await file.query_info_async(modeAttr, queryFlags, ioPriority, null);
 
-                    info.set_attribute_uint32(modeAttr, mode);
-                    info.set_attribute_string(trustedAttr, 'yes');
-                    file.set_attributes_async (info, queryFlags, ioPriority, null,
-                        (o, res) => {
-                            o.set_attributes_finish(res);
+            let mode = info.get_attribute_uint32(modeAttr) | 0o100;
+            info.set_attribute_uint32(modeAttr, mode);
+            info.set_attribute_string(trustedAttr, 'yes');
+            await file.set_attributes_async(info, queryFlags, ioPriority, null);
 
-                            // Hack: force nautilus to reload file info
-                            this._touchFile(file);
-                        });
-                } catch (e) {
-                    log(`Failed to mark file as trusted: ${e.message}`);
-                }
-            });
+            // Hack: force nautilus to reload file info
+            info = new Gio.FileInfo();
+            info.set_attribute_uint64(Gio.FILE_ATTRIBUTE_TIME_ACCESS,
+                                      GLib.get_real_time());
+            try {
+                await file.set_attributes_async(info, queryFlags, ioPriority, null);
+            } catch (e) {
+                log(`Failed to update access time: ${e.message}`);
+            }
+        } catch (e) {
+            log(`Failed to mark file as trusted: ${e.message}`);
+        }
     }
 
     destroy() {
@@ -399,9 +390,11 @@ class ApplicationsButton extends PanelMenu.Button {
 
         let hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
 
-        this._label = new St.Label({ text: _("Applications"),
-                                     y_expand: true,
-                                     y_align: Clutter.ActorAlign.CENTER });
+        this._label = new St.Label({
+            text: _('Applications'),
+            y_expand: true,
+            y_align: Clutter.ActorAlign.CENTER
+        });
         hbox.add_child(this._label);
         hbox.add_child(PopupMenu.arrowIcon(St.Side.BOTTOM));
 
@@ -457,8 +450,10 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 
     _createVertSeparator() {
-        let separator = new St.DrawingArea({ style_class: 'calendar-vertical-separator',
-                                             pseudo_class: 'highlighted' });
+        let separator = new St.DrawingArea({
+            style_class: 'calendar-vertical-separator',
+            pseudo_class: 'highlighted'
+        });
         separator.connect('repaint', this._onVertSepRepaint.bind(this));
         return separator;
     }
@@ -470,12 +465,12 @@ class ApplicationsButton extends PanelMenu.Button {
         this._tree.disconnect(this._treeChangedId);
         this._tree = null;
 
+        let handler = Main.sessionMode.hasOverview ?
+            Main.overview.toggle.bind(Main.overview) : null;
         Main.wm.setCustomKeybindingHandler('panel-main-menu',
                                            Shell.ActionMode.NORMAL |
                                            Shell.ActionMode.OVERVIEW,
-                                           Main.sessionMode.hasOverview ?
-                                           Main.overview.toggle.bind(Main.overview) :
-                                           null);
+                                           handler);
 
         this._desktopTarget.destroy();
     }
@@ -491,8 +486,8 @@ class ApplicationsButton extends PanelMenu.Button {
     _onMenuKeyPress(actor, event) {
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.KEY_Left || symbol == Clutter.KEY_Right) {
-            let direction = symbol == Clutter.KEY_Left ? Gtk.DirectionType.LEFT
-                                                       : Gtk.DirectionType.RIGHT;
+            let direction = symbol == Clutter.KEY_Left ?
+                Gtk.DirectionType.LEFT : Gtk.DirectionType.RIGHT;
             if (this.menu.actor.navigate_focus(global.stage.key_focus, direction, false))
                 return true;
         }
@@ -529,7 +524,7 @@ class ApplicationsButton extends PanelMenu.Button {
         Main.wm.setCustomKeybindingHandler('panel-main-menu',
                                            Shell.ActionMode.NORMAL |
                                            Shell.ActionMode.OVERVIEW,
-                                           () => { this.menu.toggle(); });
+                                           () => this.menu.toggle());
     }
 
     _redisplay() {
@@ -600,9 +595,12 @@ class ApplicationsButton extends PanelMenu.Button {
         this.menu.addMenuItem(section);
         this.mainBox = new St.BoxLayout({ vertical: false });
         this.leftBox = new St.BoxLayout({ vertical: true });
-        this.applicationsScrollBox = new St.ScrollView({ x_fill: true, y_fill: false,
-                                                         y_align: St.Align.START,
-                                                         style_class: 'apps-menu vfade' });
+        this.applicationsScrollBox = new St.ScrollView({
+            x_fill: true,
+            y_fill: false,
+            y_align: St.Align.START,
+            style_class: 'apps-menu vfade'
+        });
         this.applicationsScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         let vscroll = this.applicationsScrollBox.get_vscroll_bar();
         vscroll.connect('scroll-start', () => {
@@ -611,21 +609,30 @@ class ApplicationsButton extends PanelMenu.Button {
         vscroll.connect('scroll-stop', () => {
             this.menu.passEvents = false;
         });
-        this.categoriesScrollBox = new St.ScrollView({ x_fill: true, y_fill: false,
-                                                       y_align: St.Align.START,
-                                                       style_class: 'vfade' });
+        this.categoriesScrollBox = new St.ScrollView({
+            x_fill: true,
+            y_fill: false,
+            y_align: St.Align.START,
+            style_class: 'vfade'
+        });
         this.categoriesScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         vscroll = this.categoriesScrollBox.get_vscroll_bar();
-        vscroll.connect('scroll-start', () => { this.menu.passEvents = true; });
-        vscroll.connect('scroll-stop', () => { this.menu.passEvents = false; });
-        this.leftBox.add(this.categoriesScrollBox, { expand: true,
-                                                     x_fill: true, y_fill: true,
-                                                     y_align: St.Align.START });
+        vscroll.connect('scroll-start', () => this.menu.passEvents = true);
+        vscroll.connect('scroll-stop', () => this.menu.passEvents = false);
+        this.leftBox.add(this.categoriesScrollBox, {
+            expand: true,
+            x_fill: true,
+            y_fill: true,
+            y_align: St.Align.START
+        });
 
         let activities = new ActivitiesMenuItem(this);
-        this.leftBox.add(activities.actor, { expand: false,
-                                             x_fill: true, y_fill: false,
-                                             y_align: St.Align.START });
+        this.leftBox.add(activities.actor, {
+            expand: false,
+            x_fill: true,
+            y_fill: false,
+            y_align: St.Align.START
+        });
 
         this.applicationsBox = new St.BoxLayout({ vertical: true });
         this.applicationsScrollBox.add_actor(this.applicationsBox);
@@ -633,8 +640,16 @@ class ApplicationsButton extends PanelMenu.Button {
         this.categoriesScrollBox.add_actor(this.categoriesBox);
 
         this.mainBox.add(this.leftBox);
-        this.mainBox.add(this._createVertSeparator(), { expand: false, x_fill: false, y_fill: true });
-        this.mainBox.add(this.applicationsScrollBox, { expand: true, x_fill: true, y_fill: true });
+        this.mainBox.add(this._createVertSeparator(), {
+            expand: false,
+            x_fill: false,
+            y_fill: true
+        });
+        this.mainBox.add(this.applicationsScrollBox, {
+            expand: true,
+            x_fill: true,
+            y_fill: true
+        });
         section.actor.add_actor(this.mainBox);
     }
 
@@ -652,17 +667,19 @@ class ApplicationsButton extends PanelMenu.Button {
         let iter = root.iter();
         let nextType;
         while ((nextType = iter.next()) != GMenu.TreeItemType.INVALID) {
-            if (nextType == GMenu.TreeItemType.DIRECTORY) {
-                let dir = iter.get_directory();
-                if (!dir.get_is_nodisplay()) {
-                    let categoryId = dir.get_menu_id();
-                    this.applicationsByCategory[categoryId] = [];
-                    this._loadCategory(categoryId, dir);
-                    if (this.applicationsByCategory[categoryId].length > 0) {
-                        let categoryMenuItem = new CategoryMenuItem(this, dir);
-                        this.categoriesBox.add_actor(categoryMenuItem.actor);
-                    }
-                }
+            if (nextType != GMenu.TreeItemType.DIRECTORY)
+                continue;
+
+            let dir = iter.get_directory();
+            if (dir.get_is_nodisplay())
+                continue;
+
+            let categoryId = dir.get_menu_id();
+            this.applicationsByCategory[categoryId] = [];
+            this._loadCategory(categoryId, dir);
+            if (this.applicationsByCategory[categoryId].length > 0) {
+                let categoryMenuItem = new CategoryMenuItem(this, dir);
+                this.categoriesBox.add_actor(categoryMenuItem.actor);
             }
         }
 
@@ -691,22 +708,20 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 
     _displayButtons(apps) {
-        if (apps) {
-            for (let i = 0; i < apps.length; i++) {
-                let app = apps[i];
-                let item;
-                if (app instanceof Shell.App)
-                    item = this._applicationsButtons.get(app);
-                else
-                    item = new PopupMenu.PopupSeparatorMenuItem();
-                if (!item) {
-                    item = new ApplicationMenuItem(this, app);
-                    item.setDragEnabled(this._desktopTarget.hasDesktop);
-                    this._applicationsButtons.set(app, item);
-                }
-                if (!item.actor.get_parent())
-                    this.applicationsBox.add_actor(item.actor);
+        for (let i = 0; i < apps.length; i++) {
+            let app = apps[i];
+            let item;
+            if (app instanceof Shell.App)
+                item = this._applicationsButtons.get(app);
+            else
+                item = new PopupMenu.PopupSeparatorMenuItem();
+            if (!item) {
+                item = new ApplicationMenuItem(this, app);
+                item.setDragEnabled(this._desktopTarget.hasDesktop);
+                this._applicationsButtons.set(app, item);
             }
+            if (!item.actor.get_parent())
+                this.applicationsBox.add_actor(item.actor);
         }
     }
 
