@@ -50,11 +50,12 @@ function flashMessage(message) {
     text.set_position(monitor.x + Math.floor(monitor.width / 2 - text.width / 2),
                       monitor.y + Math.floor(monitor.height / 2 - text.height / 2));
 
-    Tweener.addTween(text,
-                     { opacity: 0,
-                       time: MESSAGE_FADE_TIME,
-                       transition: 'easeOutQuad',
-                       onComplete: hideMessage });
+    Tweener.addTween(text, {
+        opacity: 0,
+        time: MESSAGE_FADE_TIME,
+        transition: 'easeOutQuad',
+        onComplete: hideMessage
+    });
 }
 
 let SIZES = [
@@ -72,8 +73,8 @@ function cycleScreenshotSizes(display, window, binding) {
     let backwards = (modifiers & Meta.VirtualModifier.SHIFT_MASK) != 0;
 
     // Unmaximize first
-    if (window.maximized_horizontally || window.maximizedVertically)
-        window.unmaximize(Meta.MaximizeFlags.HORIZONTAL | Meta.MaximizeFlags.VERTICAL);
+    if (window.get_maximized() != 0)
+        window.unmaximize(Meta.MaximizeFlags.BOTH);
 
     let workArea = window.get_work_area_current_monitor();
     let outerRect = window.get_frame_rect();
